<?php

namespace App\Http\Controllers\BackEnd;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Validation\Rule;
use App\Models\Admin;
use App\Models\Role;
use Illuminate\Support\Facades\Response;
use Illuminate\Support\Facades\Session;
use Illuminate\Support\Facades\Validator;

class UserController extends Controller
{
    public function index()
    {
        $data['users'] = Admin::all();
        $data['roles'] = Role::all();
        return view('backend.user.index', $data);
    }

    public function edit($id)
    {
        $data['user'] = Admin::where('id', $id)->firstOrFail();
        $data['roles'] = Role::all();
        return view('backend.user.edit', $data);
    }

    public function store(Request $request)
    {
        $image = $request->image;
        $allowedExts = array('jpg', 'png', 'jpeg', 'svg');

        $rules = [
            'image' => 'required',
            'username' => 'required|max:255|unique:admins',
            'email' => 'required|email|max:255|unique:admins',
            'first_name' => 'required|max:255',
            'last_name' => 'required|max:255',
            'password' => 'required|confirmed',
            'role' => 'required',
        ];
        if ($request->hasFile('image')) {
            $extImage = $request->file('image')->getClientOriginalExtension();
            $rules['image'] = [
                function ($attribute, $value, $fail) use ($extImage, $allowedExts) {
                    if (!in_array($extImage, $allowedExts)) {
                        return $fail("Only png, jpg, jpeg, svg image is allowed");
                    }
                }
            ];
        }
        $validator = Validator::make($request->all(), $rules);
        if ($validator->fails()) {
            return Response::json(
                [
                    'errors' => $validator->getMessageBag()->toArray()
                ],
                400
            );
        }

        $user = new Admin;
        $user->role_id = $request->role;
        $user->username = $request->username;
        $user->email = $request->email;
        $user->first_name = $request->first_name;
        $user->last_name = $request->last_name;
        $user->password = bcrypt($request->password);

        if ($request->hasFile('image')) {
            $filename = time() . '.' . $request->file('image')->getClientOriginalExtension();
            $directory = public_path('assets/img/admins/');
            @mkdir($directory, 0775, true);
            $request->file('image')->move($directory, $filename);
            $user->image = $filename;
        }


        $user->save();

        Session::flash('success', 'User created successfully!');
        return "success";
    }


    public function update(Request $request)
    {
        $user = Admin::where('id', $request->user_id)->first();
        $image = $request->image;
        $allowedExts = array('jpg', 'png', 'jpeg', 'svg');

        $rules = [
            'username' => [
                'required',
                'max:255',
                Rule::unique('admins')->ignore($user->id),
            ],
            'email' => [
                'required',
                'email',
                'max:255',
                Rule::unique('admins')->ignore($user->id),
            ],
            'first_name' => 'required|max:255',
            'last_name' => 'required|max:255',
            'role' => 'required',
        ];

        if ($request->hasFile('image')) {
            $extImage = $request->file('image')->getClientOriginalExtension();
            $rules['image'] = [
                function ($attribute, $value, $fail) use ($extImage, $allowedExts) {
                    if (!in_array($extImage, $allowedExts)) {
                        return $fail("Only png, jpg, jpeg, svg image is allowed");
                    }
                }
            ];
        }

        $validator = Validator::make($request->all(), $rules);
        if ($validator->fails()) {
            return Response::json(
                [
                    'errors' => $validator->getMessageBag()->toArray()
                ],
                400
            );
        }

        $user->username = $request->username;
        $user->email = $request->email;
        $user->first_name = $request->first_name;
        $user->last_name = $request->last_name;
        $user->status = $request->status;
        $user->role_id = $request->role;

        if ($request->hasFile('image')) {
            @unlink(public_path('assets/img/admins/') . $user->image);
            $filename = time() . '.' . $request->file('image')->getClientOriginalExtension();
            $directory = public_path('assets/img/admins/');
            @mkdir($directory, 0775, true);
            $request->file('image')->move($directory, $filename);
            $user->image = $filename;
        }

        $user->save();

        Session::flash('success', 'User updated successfully!');
        return "success";
    }

    public function delete(Request $request)
    {
        if ($request->user_id == 1) {
            Session::flash('warning', 'You cannot delete the owner!');
            return back();
        }

        $user = Admin::where('id', $request->user_id)->first();
        @unlink(public_path('assets/img/admins/') . $user->image);
        $user->delete();

        Session::flash('success', 'User deleted successfully!');
        return back();
    }

    public function managePermissions($id)
    {
        $data['user'] = Admin::where('id', $id)->first();
        return view('backend.user.permission.manage', $data);
    }

    public function updatePermissions(Request $request)
    {
        $permissions = json_encode($request->permissions);
        $user = Admin::where('id', $request->user_id)->first();
        $user->permissions = $permissions;
        $user->save();

        Session::flash('success', "Permissions updated successfully for '$user->name' user");
        return back();
    }
}
